package javawrapper;


import java.io.FileWriter;
import java.io.File;

/**
 * Tests the HarvestInterface interface and file writing.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestHarvestInterface extends ModelTestCase {

  /**
   * Tests the harvest interface.
   */
  public void testAll() {
    GUIManager oManager = null;
    String sFileName = null;
    String sFileName2 = null;
    try {

      MainWindow oMainWindow = new MainWindow();
      oManager = oMainWindow.m_oDataManager;
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      TreePopulation oPop = oManager.GetTreePopulation();

      //Call the interface, and fill it in (but we won't be displaying it)
      HarvestInterfaceSetup oWindow = new HarvestInterfaceSetup(null, oManager);
      oWindow.m_jExecutablePathEdit.setText("c:\\code\\core_model\\coremodel.exe");
      oWindow.m_jSORTIEOutFileEdit.setText("c:\\sortieout.txt");
      oWindow.m_jExecutableHarvestOutFileEdit.setText("c:\\execout.txt");
      oWindow.m_jHarvestPeriod.setText("2");
      //We can't do behavior copies through the interface, so do it this way
      oWindow.m_oBehaviorCopy.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oWindow.m_oBehaviorCopy.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, 3, oPop));
      oWindow.m_jFileColumnsListModel.addElement("Light");
      oWindow.m_jFileColumnsListModel.addElement("new 1");
      oWindow.m_jFileColumnsListModel.addElement("new 2");
      oWindow.m_jBatchParamsFileEdit.setText("c:\\code\\core_model\\SimManager.cpp");
      oWindow.m_jBatchSingleRunParamsFileEdit.setText("c:\\singlebatchparams.txt");
      oWindow.m_jExecutableArgs.setText("args 1 args 2");
      oWindow.m_jExecutableUpdateOutFileEdit.setText("c:\\sortieupdate.txt");
      oWindow.m_jNewTreeDataMembersListModel.addElement("new 1");
      oWindow.m_jNewTreeDataMembersListModel.addElement("new 2");
      oWindow.actionPerformed(new java.awt.event.ActionEvent(oWindow, 0, "OK"));

      //Make sure the data got transferred to the disturbance behaviors
      DisturbanceBehaviors oDisturbance = oManager.GetDisturbanceBehaviors();
      assertTrue(oDisturbance.m_sHarvIntExecutable.GetValue().equals("c:\\code\\core_model\\coremodel.exe"));
      assertTrue(oDisturbance.m_sHarvIntSORTIEOutFile.GetValue().equals("c:\\sortieout.txt"));
      assertTrue(oDisturbance.m_sHarvIntExecHarvestOutFile.GetValue().equals("c:\\execout.txt"));
      assertEquals(2, oDisturbance.m_iHarvIntHarvestPeriod.GetValue());
      Behavior oBeh = oDisturbance.GetBehaviorByKey("Harvest Interface");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(2, oBeh.GetNumberOfCombos());
      assertEquals(1, oBeh.GetSpeciesTypeCombo(0).GetSpecies());
      assertEquals(3, oBeh.GetSpeciesTypeCombo(0).GetType());
      assertEquals(2, oBeh.GetSpeciesTypeCombo(1).GetSpecies());
      assertEquals(3, oBeh.GetSpeciesTypeCombo(1).GetType());
      assertEquals(3, oDisturbance.mp_sHarvIntFileColumns.GetValue().size());
      assertEquals("Light", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(0)));
      assertEquals("new 1", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(1)));
      assertEquals("new 2", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(2)));
      assertTrue(oDisturbance.m_sHarvIntBatchParamsFile.GetValue().equals("c:\\code\\core_model\\SimManager.cpp"));
      assertTrue(oDisturbance.m_sHarvIntBatchSingleRunParamsFile.GetValue().equals("c:\\singlebatchparams.txt"));
      assertTrue(oDisturbance.m_sHarvIntExecArgs.GetValue().equals("args 1 args 2"));
      assertTrue(oDisturbance.m_sHarvIntExecUpdateOutFile.GetValue().equals("c:\\sortieupdate.txt"));
      assertEquals(2, oDisturbance.mp_sHarvIntNewTreeDataMembers.size());
      assertEquals("new 1", String.valueOf(oDisturbance.mp_sHarvIntNewTreeDataMembers.elementAt(0)));
      assertEquals("new 2", String.valueOf(oDisturbance.mp_sHarvIntNewTreeDataMembers.elementAt(1)));

      //Write an XML file and read it back in, and make sure all the settings
      //are correct
      sFileName2 = "secondtest.xml";
      oManager.WriteParameterFile(sFileName2);
      oMainWindow = null;
      oMainWindow = new MainWindow();
      oManager = oMainWindow.m_oDataManager;
      oManager.InputXMLParameterFile(sFileName2);
      oDisturbance = oManager.GetDisturbanceBehaviors();
      assertTrue(oDisturbance.m_sHarvIntExecutable.GetValue().equals("c:\\code\\core_model\\coremodel.exe"));
      assertTrue(oDisturbance.m_sHarvIntSORTIEOutFile.GetValue().equals("c:\\sortieout.txt"));
      assertTrue(oDisturbance.m_sHarvIntExecHarvestOutFile.GetValue().equals("c:\\execout.txt"));
      assertEquals(2, oDisturbance.m_iHarvIntHarvestPeriod.GetValue());
      oBeh = oDisturbance.GetBehaviorByKey("Harvest Interface");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(2, oBeh.GetNumberOfCombos());
      assertEquals(1, oBeh.GetSpeciesTypeCombo(0).GetSpecies());
      assertEquals(3, oBeh.GetSpeciesTypeCombo(0).GetType());
      assertEquals(2, oBeh.GetSpeciesTypeCombo(1).GetSpecies());
      assertEquals(3, oBeh.GetSpeciesTypeCombo(1).GetType());
      assertEquals(3, oDisturbance.mp_sHarvIntFileColumns.GetValue().size());
      assertEquals("Light", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(0)));
      assertEquals("new 1", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(1)));
      assertEquals("new 2", String.valueOf(oDisturbance.mp_sHarvIntFileColumns.GetValue().elementAt(2)));
      assertTrue(oDisturbance.m_sHarvIntBatchParamsFile.GetValue().equals("c:\\code\\core_model\\SimManager.cpp"));
      assertTrue(oDisturbance.m_sHarvIntBatchSingleRunParamsFile.GetValue().equals("c:\\singlebatchparams.txt"));
      assertTrue(oDisturbance.m_sHarvIntExecArgs.GetValue().equals("args 1 args 2"));
      assertTrue(oDisturbance.m_sHarvIntExecUpdateOutFile.GetValue().equals("c:\\sortieupdate.txt"));
      assertEquals(2, oDisturbance.mp_sHarvIntNewTreeDataMembers.size());
      assertEquals("new 1", String.valueOf(oDisturbance.mp_sHarvIntNewTreeDataMembers.elementAt(0)));
      assertEquals("new 2", String.valueOf(oDisturbance.mp_sHarvIntNewTreeDataMembers.elementAt(1)));
    }
    catch (ModelException oErr) {
      fail("Disturbance validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
      new File(sFileName2).delete();
    }

    System.out.println("Storm ValidateData test was successful.");
  }

  /**
   * Makes sure the correct options are displayed in parameters.
   */
  public void testFormatDataForDisplay() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      Behavior oBeh;

      //Case:  no disturbance behaviors are enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      TreePopulation oPop = oManager.GetTreePopulation();
      DisturbanceBehaviors oDisturbance = oManager.GetDisturbanceBehaviors();
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Harvest is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Harvest");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Natural Disturbance is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Episodic Mortality");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Harvest Interface is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Harvest Interface");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      System.out.println("FormatDataForDisplay succeeded for disturbance.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a file with no disturbance behaviors.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNeutralFile() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
